package com.sencorsta.urlMaster.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.sencorsta.ids.core.entity.ErrorCode;
import com.sencorsta.ids.core.entity.FileUpload;
import com.sencorsta.ids.core.entity.IdsRequest;
import com.sencorsta.ids.core.entity.IdsResponse;
import com.sencorsta.ids.core.entity.annotation.Autowired;
import com.sencorsta.ids.core.entity.annotation.Controller;
import com.sencorsta.ids.core.entity.annotation.RequestMapping;
import com.sencorsta.urlMaster.entity.HelloFileRequest;
import com.sencorsta.urlMaster.service.UrlMasterService;
import com.sencorsta.utils.object.Jsons;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.FileOutputStream;
import java.util.List;

/**
 * @author ICe
 */
@Controller
@RequestMapping("/urlMaster")
@Slf4j
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class UrlMasterController {

    private final UrlMasterService urlMasterService;

    @RequestMapping("/helloWorld")
    public IdsResponse<String> helloWorld(IdsRequest<byte[]> request) {
        return new IdsResponse<>(urlMasterService.helloWorld(new String(request.getData()),request.getChannel()));
    }

    @RequestMapping("/helloUpLoad")
    public IdsResponse<ObjectNode> helloUpLoad(IdsRequest<HelloFileRequest> request){
        HelloFileRequest data = request.getData();
        List<FileUpload> files = data.getFiles();

        // 用于存储每个文件的 MD5 码和文件名
        ObjectNode result = Jsons.createObjectNode();
        try {
            for (FileUpload file : files) {
                String fileName = file.getName();
                byte[] fileData = file.getData();

                // 计算文件的 MD5 码
                String md5 = DigestUtil.md5Hex(fileData);
                // 保存文件到本地
                saveFileToLocal(md5, fileData);
                // 将文件名和对应的 MD5 码加入结果中
                result.put(fileName, md5);
            }
            // 返回结果
            return new IdsResponse<>(result);
        } catch (Exception e) {
            e.printStackTrace();
            // 处理异常情况
            return new IdsResponse<>(null,new ErrorCode(500,"失败"));
        }
    }

    /**
     * 保存文件到本地
     *
     * @param fileName fileName
     * @param fileData fileData
     */
    private void saveFileToLocal(String fileName, byte[] fileData) {
        String workingDir = System.getProperty("user.dir");
        String uploadDir = workingDir + File.separator + "web" + File.separator + "oss";

        // 创建目录（如果不存在）
        FileUtil.mkdir(uploadDir);

        String filePath = uploadDir + File.separator + fileName;
        File file = new File(filePath);
        if (!file.exists()) {
            try (FileOutputStream fos = new FileOutputStream(file)) {
                IoUtil.write(fos, true, fileData);
                log.info("File saved: " + filePath);
            } catch (Exception e) {
                log.error("File upload Exception: " + filePath, e);
            }
        } else {
            log.debug("File already exists, skipping: " + filePath);
        }
    }

}
